/*
 * Copyright 2009 Extreme Engineering Solutions, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

/*
 * Include additional files required for supporting different operating systems
 */
#include "compiler.h"
#ifdef __MINGW32__
#include "mingw_support.c"
#endif
#if defined(__APPLE__) && __DARWIN_C_LEVEL < 200809L
#include "getline.c"
#endif
#ifdef __riscos__
#include <stdio.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <stddef.h>

typedef struct {
	int fd;
	int prot;
	size_t len;
	int offset;
	char buf[0];
} mmap_struct;

__caddr_t fake_mmap(__caddr_t addr, size_t len, int prot, int flags, int fd, off_t offset)
{
	if((prot & PROT_EXEC) || (prot == PROT_NONE))
	{
		errno = ENOSYS;
		return MAP_FAILED;
	}
	mmap_struct *mm = (mmap_struct *) malloc(sizeof(mmap_struct)+len);
	if(!mm)
	{
		errno = ENOMEM;
		return MAP_FAILED;
	}
	mm->fd = fd;
	mm->prot = prot;
	mm->len = len;
	mm->offset = offset;
	if(prot & PROT_READ)
	{
		__off_t curpos = lseek(fd,0,SEEK_CUR);
		lseek(fd,offset,SEEK_SET);
		read(fd,mm->buf,len);
		lseek(fd,curpos,SEEK_SET);
	}
	return mm->buf;
}

int fake_munmap(__caddr_t addr,size_t len)
{
	mmap_struct *mm = (mmap_struct *) (((int)addr)-offsetof(mmap_struct,buf));
	if(mm->prot & PROT_WRITE)
	{
		__off_t curpos = lseek(mm->fd,0,SEEK_CUR);
		lseek(mm->fd,mm->offset,SEEK_SET);
		write(mm->fd,mm->buf,len);
		lseek(mm->fd,curpos,SEEK_SET);
	}
	free(mm);
	return 0;
}
#endif
